import 'package:flutter/services.dart';
import 'package:bg_flutter_channel/channel/channel_callback.dart';
import 'package:bg_flutter_channel/channel/model/action_arguments.dart';

import 'package:bg_flutter_channel/channel/constant.dart';
import 'dart:convert';

import 'package:bg_flutter_channel/channel/model/channel_result.dart';
import 'package:bg_flutter_channel/channel/model/http_arguments.dart';
import 'package:bg_flutter_channel/channel/model/scheme_arguments.dart';

/// channel的交互协议
/// 目前主要有三种method，分别是http,scheme,action，
/// 分别对应的方法是requestHttpMethod，requestSchemeMethod，requestActionMethod
/// 第一种是http网络请求使用
/// 第二种是路由跳转使用
/// 第三种是事件点击使用
/// 当然还可以用自定义的方式sendMethodToNativeAsync，不过不建议，因为都走自定义的话后期跨端维护麻烦
/// 每种方式的method，请求的类都不一样，HttpArguments，SchemeArguments，ActionArguments
/// 然后响应的结果体都是这种格式ChannelResult
/// 也就是说原生那边的请求和响应处理也是用这几个实体类
/// 比如：发起一个网络请求
/// 发送：
/// 接收：{\"code\":\"200\",\"message\":\"success\",\"result\":\"data\"}
class Channel {
  // 渠道交互 flutter -> 原生
  static const String method_common = "com.banggood.flutter/methodsChannel";

  // 渠道交互 原生 -> flutter
  static const String event = "com.banggood.flutter/eventChannel";


  // 渠道交互 flutter -> 原生
  static const methodsChannel = const MethodChannel(method_common);

  // 渠道交互 原生 -> flutter
  static const eventChannel = const EventChannel(event);

  /// 发送通信到原生
  static sendMethodToNative(String method, [dynamic arguments]) {
    methodsChannel.invokeMethod(method, arguments);
  }

  /// 可自定的请求，慎用
  static Future<ChannelResult> sendMethodToNativeAsync(String method, dynamic arguments, [ChannelCallback callback]) async{

    dynamic result = await methodsChannel.invokeMethod(method, arguments);
    //dynamic result = "{\"code\":\"200\",\"message\":\"success\",\"result\":\"data\"}";
    print("flutter receive result:" + result);

    ChannelResult channelResult;
    if(callback != null && result is String){
      channelResult = ChannelResult.fromJson(json.decode(result));
      if(channelResult.code == ChannelConstant.channelSuccess){
        callback.onSuccess(channelResult, channelResult.result??"");
      } else {
        callback.onFail(channelResult, channelResult.result??"");
      }
    }

    return channelResult;
  }

  /// 网络的请求
  static Future<ChannelResult> requestHttpMethod(String serverPath, Map header, Map params, [ChannelCallback callback]) async{

    var httpArguments = new HttpArguments(serverPath, header, params, false);
    dynamic channelResult = await sendMethodToNativeAsync(ChannelConstant.methodHttp, httpArguments.toJson(), callback);
    return channelResult;
  }
  /// 网络请求，并且显示loading效果
  static Future<ChannelResult> requestHttpMethodWithLoading(String serverPath, Map header, Map params, [ChannelCallback callback]) async{

    var httpArguments = new HttpArguments(serverPath, header, params, true);
    dynamic channelResult = await sendMethodToNativeAsync(ChannelConstant.methodHttp, httpArguments.toJson(), callback);
    return channelResult;
  }

  /// 路由的请求
  static Future<ChannelResult> requestSchemeMethod(String path, [ChannelCallback callback]) async{

    var schemeArguments = new SchemeArguments(path);
    dynamic channelResult = await sendMethodToNativeAsync(ChannelConstant.methodScheme, schemeArguments.toJson(), callback);
    return channelResult;
  }

  /// 行为的请求
  static Future<ChannelResult> requestActionMethod(String event, dynamic data, [ChannelCallback callback]) async{

    var schemeArguments = new ActionArguments(event, data);
    dynamic channelResult = await sendMethodToNativeAsync(ChannelConstant.methodAction, schemeArguments.toJson(), callback);
    return channelResult;
  }


}
